@@ -4,7 +4,8 @@ max <- function(..., na.rm = TRUE) suppressWarnings(base::max(..., na.rm = na.rm
 arrows <- function(...) suppressWarnings(graphics::arrows(...))
 
 hist.numeric <- function(x, ...) {
-  if(all(!diff(x))) {
+  if(all(!diff(signif(x, 3)))) {
+    x <- signif(x, 3)
     breaks <- c(0.99, 1.01) * x[1]
     xlim <- c(0.8, 1.2) * x[1]
     graphics::hist.default(x, breaks = breaks, xlim = xlim, ...)
@@ -133,10 +134,16 @@ optimize_TMB_model <- function(obj, control = list(), use_hessian = FALSE, resta
   restart <- as.integer(restart)
   if(is.null(obj$env$random) && use_hessian) h <- obj$he else h <- NULL
   low <- rep(-Inf, length(obj$par))
+  upr <- rep(Inf, length(obj$par))
+  
   if(any(c("U_equilibrium", "F_equilibrium") %in% names(obj$par))) {
     low[match(c("U_equilibrium", "F_equilibrium"), names(obj$par))] <- 0
   }
-  opt <- tryCatch(suppressWarnings(nlminb(obj$par, obj$fn, obj$gr, h, control = control, lower = low)),
+  if(!is.null(obj$env$data$use_prior[1]) && obj$env$data$use_prior[1] > 0) { # Uniform priors need bounds
+    low[names(obj$par) == "R0x"] <- log(obj$env$data$prior_dist[1, 1]) + log(obj$env$data$rescale)
+    upr[names(obj$par) == "R0x"] <- log(obj$env$data$prior_dist[1, 2]) + log(obj$env$data$rescale)
+  }
+  opt <- tryCatch(suppressWarnings(nlminb(obj$par, obj$fn, obj$gr, h, control = control, lower = low, upper = upr)),
                   error = function(e) as.character(e))
   SD <- get_sdreport(obj)
 
@@ -360,12 +367,29 @@ make_prior <- function(prior, nsurvey, SR_rel, dots = list(), msg = TRUE) { # lo
   if(no_index) nsurvey <- 1 # Use only on next two lines
   use_prior <- rep(0L, nsurvey + 3)
   pr_matrix <- matrix(NA_real_, nsurvey + 3, 2) %>% 
-    structure(dimnames = list(c("log_R0", "h", "log_M", paste0("q_", 1:nsurvey)), c("par1", "par2")))
+    structure(dimnames = list(c("R0", "h", "log_M", paste0("q_", 1:nsurvey)), c("par1", "par2")))
   
   if(!is.null(prior$R0)) {
-    if(msg) message("Prior for log_R0 found.")
-    use_prior[1] <- 1L
-    pr_matrix[1, ] <- c(log(prior$R0[1]), prior$R0[2])
+    if(length(prior$R0) == 2) prior$R0 <- c(1, prior$R0) # Backwards compatibility
+    use_prior[1] <- as.integer(prior$R0[1]) # 1 - lognormal, 2 - uniform on log-R0, 3 - uniform on R0
+    
+    if(msg) {
+      message(
+        switch(
+          use_prior[1],
+          "1" = "Lognormal prior for R0 found.",
+          "2" = "Uniform prior for log(R0) found.",
+          "3" = "Uniform prior for R0 found."
+        )
+      )
+    }
+    
+    if(use_prior[1] == 1) {
+      pr_matrix[1, ] <- c(log(prior$R0[2]), prior$R0[3])
+    } else {
+      pr_matrix[1, ] <- prior$R0[2:3]
+      if(prior$R0[3] <= prior$R0[2]) stop("The upper bound of the R0 prior is less than the lower bound.")
+    }
   }
   if(!is.null(prior$h)) {
     if(msg) message("Prior for steepness (h) found.")
@@ -406,3 +430,23 @@ make_prior <- function(prior, nsurvey, SR_rel, dots = list(), msg = TRUE) { # lo
   }
   return(list(use_prior = use_prior, pr_matrix = pr_matrix))
 }
+
+
+solve_F <- function(N, M, plusgroup = TRUE) {
+  FM <- array(NA_real_, dim(M))
+  nyears <- nrow(M)
+  n_age <- ncol(M)
+  for(y in 1:nyears) {
+    for(a in 3:n_age - 2) FM[y, a] <- -log(N[y+1,a+1]/N[y,a]/exp(-M[y,a]))
+    if(plusgroup) {
+      FM[y,n_age] <- FM[y,n_age-1] <- -log(N[y+1,n_age]/(N[y,n_age] * exp(-M[y,n_age]) + 
+                                                           N[y,n_age-1] * exp(-M[y,n_age-1])))
+    } else {
+      FM[y,n_age] <- FM[y,n_age-1] <- -log(N[y+1,n_age]/N[y,n_age-1]/exp(-M[y,n_age-1]))
+    }
+  }
+  FM[is.na(FM) | FM < 0] <- 1e-8
+  return(FM)
+}
+
+
