@@ -105,8 +105,8 @@ rmd_RCM_Find <- function(fig.cap = "Apical F from RCM model. These values may be
 
 rmd_RCM_sel <- function(fig.cap = "Operating model selectivity among simulations.") {
   c(paste0("```{r, fig.cap = \"", fig.cap, "\"}"),
-    "if(nsel_block == 1) {",
-    "  vul <- do.call(cbind, lapply(report_list, getElement, \"vul_len\"))",
+    "vul <- sapply(report_list, getElement, \"vul_len\")",
+    "if(nsel_block == 1 && all(!is.na(vul))) {",
     "  matplot(RCMdata@Misc$lbinmid, vul, type = \"l\", col = \"black\",",
     "          xlab = \"Length\", ylab = \"Selectivity (last historical year)\", ylim = c(0, 1.1))",
     "} else {",
@@ -127,7 +127,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "bl_col <- gplots::rich.colors(length(bl))",
            "Year_legend <- character(length(bl))",
            "for(bb in 1:length(bl)) {",
-           "  vul_bb[[bb]] <- do.call(cbind, lapply(report_list, function(x) x$vul_len[, bl[bb]]))",
+           "  vul_bb[[bb]] <- sapply(report_list, function(x) x$vul_len[, bl[bb]])",
            paste0("  Year_legend[bb] <- Year[RCMdata@sel_block[, ", ff, "] == bl[bb]] %>% range() %>% paste(collapse = \"-\")"),
            "}",
            "test <- vapply(vul_bb, function(x) all(!is.na(x)), logical(1))",
@@ -155,7 +155,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Fishing Mortality of ", f_name[ff], ".\"}"),
-           paste0("FM <- do.call(cbind, lapply(report_list, function(x) x$F[, ", ff, "]))"),
+           paste0("FM <- sapply(report_list, function(x) x$F[, ", ff, "])"),
            paste0("matplot(Year, FM, type = \"l\", col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd, ylim = c(0, 1.1 * max(FM, na.rm = TRUE)), xlab = \"Year\", ylab = \"Fishing Mortality of ", f_name[ff], "\")"),
            "abline(h = 0, col = \"grey\")",
            "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -163,13 +163,13 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) catch from ", f_name[ff], ".\"}"),
            paste0("if(any(RCMdata@Chist[, ", ff, "] > 0)) {"),
-           paste0("  Cpred <- do.call(cbind, lapply(report_list, function(x) x$Cpred[, ", ff, "]))"),
+           paste0("  Cpred <- sapply(report_list, function(x) x$Cpred[, ", ff, "])"),
            paste0("  Chist <- RCMdata@Chist[, ", ff, "]"),
            "  ylim <- c(0.9, 1.1) * range(c(Cpred, Chist), na.rm = TRUE)",
            paste0("  plot(Year, Chist, type = \"o\", xlab = \"Year\", ylab = \"Catch of ", f_name[ff], "\", ylim = ylim)"),
            paste0("  matlines(Year, Cpred, col = scenario$col, lty = scenario$lty, lwd = scenario$lwd)"),
            "} else {",
-           paste0("  Cpred <- do.call(cbind, lapply(report_list, function(x) x$Cpred[, ", ff, "]/mean(x$Cpred[, ", ff, "])))"),
+           paste0("  Cpred <- sapply(report_list, function(x) x$Cpred[, ", ff, "]/mean(x$Cpred[, ", ff, "]))"),
            paste0("  matplot(Year, Cpred, col = scenario$col, type = \"l\", lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Predicted relative catch of ", f_name[ff], "\")"),
            "}",
            "abline(h = 0, col = \"grey\")",
@@ -177,7 +177,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) mean ages from ", f_name[ff], ".\"}"),
-           paste0("MApred <- do.call(cbind, lapply(report_list, function(x) x$CAApred[, , ", ff, "] %*% age/x$CN[, ", ff, "]))"),
+           paste0("MApred <- sapply(report_list, function(x) x$CAApred[, , ", ff, "] %*% age/x$CN[, ", ff, "])"),
            paste0("MAobs <- (RCMdata@CAA[, , ", ff, "] %*% age)/rowSums(RCMdata@CAA[, , ", ff, "], na.rm = TRUE)"),
            "ylim <- c(0.9, 1.1) * range(c(MApred, MAobs), na.rm = TRUE)",
            "matplot(Year, MApred, type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Mean age\", ylim = ylim)",
@@ -188,7 +188,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) mean lengths from ", f_name[ff], ".\"}"),
-           paste0("MLpred <- do.call(cbind, lapply(report_list, function(x) x$MLpred[, ", ff, "]))"),
+           paste0("MLpred <- sapply(report_list, function(x) x$MLpred[, ", ff, "])"),
            paste0("if(any(RCMdata@CAL[, , ", ff, "] > 0, na.rm = TRUE)) {"),
            paste0("  MLobs <- (RCMdata@CAL[, , ", ff, "] %*% RCMdata@Misc$lbinmid)/rowSums(RCMdata@CAL[, , ", ff, "], na.rm = TRUE)"),
            paste0("} else if(RCMdata@MS_type == \"length\" && any(RCMdata@MS[, ", ff, "] > 0, na.rm = TRUE)) MLobs <- RCMdata@MS[, ", ff, "] else MLobs <- NA"),
@@ -205,7 +205,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            paste0("  MWobs <- RCMdata@MS[, ", ff, "]"),
            paste0("} else MWobs <- NA"),
            "if(!all(is.na(MWobs))) {",
-           paste0("  MWpred <- do.call(cbind, lapply(report_list, function(x) x$MWpred[, ", ff, "]))"),
+           paste0("  MWpred <- sapply(report_list, function(x) x$MWpred[, ", ff, "])"),
            "  ylim <- c(0.9, 1.1) * range(c(MWpred, MWobs), na.rm = TRUE)",
            "  matplot(Year, MWpred, type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Mean weight\", ylim = ylim)",
            "  lines(Year, MWobs, col = \"black\", typ = \"o\")",
@@ -246,7 +246,7 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "",
            paste0("```{r, fig.cap = \"Selectivity of ", s_name[sur], " in last historical year.\"}"),
            "if(!is.null(report_list[[1]]$ivul)) {",
-           paste0("ivul_ff_age <- do.call(cbind, lapply(report_list, function(x) x$ivul[nyears, , ", sur, "]))"),
+           paste0("ivul_ff_age <- sapply(report_list, function(x) x$ivul[nyears, , ", sur, "])"),
            paste0("matplot(age, ivul_ff_age, type = \"l\", col = scenario$col2, xlab = \"Age\", ylim = c(0, 1), ylab = \"Selectivity of ", s_name[sur], "\")"),
            "abline(h = 0, col = \"grey\")",
            "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -254,7 +254,7 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) index values for ", s_name[sur], ".\"}"),
-           paste0("Ipred <- do.call(cbind, lapply(report_list, function(x) x$Ipred[, ", sur, "]))"),
+           paste0("Ipred <- sapply(report_list, function(x) x$Ipred[, ", sur, "])"),
            paste0("matplot(Year, Ipred, type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, ylim = c(0, 1.1 * max(c(Ipred, RCMdata@Index[, ", sur, "]), na.rm = TRUE)), xlab = \"Year\", ylab = \"", s_name[sur], "\")"),
            paste0("lines(Year, RCMdata@Index[, ", sur, "], col = \"black\", typ = \"o\")"),
            "abline(h = 0, col = \"grey\")",
@@ -266,7 +266,7 @@ rmd_RCM_index_output <- function(sur, s_name) {
            paste0("  II <- RCMdata@Index[, ", sur, "]"),
            "  ind <- seq(min(which(!is.na(II))), max(which(!is.na(II))), 1)",
            paste0("  err <- exp(log(II) + outer(RCMdata@I_sd[, ", sur, "], c(-1.96, 1.96)))"),
-           paste0("  matplot(Year[ind], Ipred[ind, ], type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, ylim = c(0, 1.1 * max(c(Ipred[ind, ], II[ind], err[ind, ]), na.rm = TRUE)), xlab = \"Year\", ylab = \"", s_name[sur], "\")"),
+           paste0("  matplot(Year[ind], Ipred[ind, , drop = FALSE], type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, ylim = c(0, 1.1 * max(c(Ipred[ind, ], II[ind], err[ind, ]), na.rm = TRUE)), xlab = \"Year\", ylab = \"", s_name[sur], "\")"),
            "  points(Year[ind], II[ind], lwd = 3, pch = 16)",
            "  arrows(Year[ind], y0 = err[ind, 1], y1 = err[ind, 2], length = 0, lwd = 1.5)",
            "  abline(h = 0, col = \"grey\")",
@@ -276,7 +276,7 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) mean ages from ", s_name[sur], ".\"}"),
            paste0("if(length(RCMdata@IAA) && any(RCMdata@IAA[, , ", sur, "] > 0, na.rm = TRUE)) {"),
-           paste0("MApred <- do.call(cbind, lapply(report_list, function(x) x$IAApred[, , ", sur, "] %*% age/rowSums(x$N[1:nyears, ] * x$ivul[1:nyears, , ", sur, "])))"),
+           paste0("MApred <- sapply(report_list, function(x) x$IAApred[, , ", sur, "] %*% age/rowSums(x$IAApred[, , ", sur, "]))"),
            paste0("MAobs <- (RCMdata@IAA[, , ", sur, "] %*% age)/rowSums(RCMdata@IAA[, , ", sur, "], na.rm = TRUE)"),
            "ylim <- c(0.9, 1.1) * range(c(MApred, MAobs), na.rm = TRUE)",
            "matplot(Year, MApred, type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Mean age\", ylim = ylim)",
@@ -289,7 +289,7 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) mean lengths from ", s_name[sur], ".\"}"),
            paste0("if(length(RCMdata@IAL) && any(RCMdata@IAL[, , ", sur, "] > 0, na.rm = TRUE)) {"),
-           paste0("MLpred <- do.call(cbind, lapply(report_list, function(x) x$IALpred[, , ", sur, "] %*% RCMdata@Misc$lbinmid/rowSums(x$N[1:nyears, ] * x$ivul[1:nyears, , ", sur, "])))"),
+           paste0("MLpred <- sapply(report_list, function(x) x$IALpred[, , ", sur, "] %*% RCMdata@Misc$lbinmid/rowSums(x$IAApred[, , ", sur, "]))"),
            paste0("MLobs <- (RCMdata@IAL[, , ", sur, "] %*% RCMdata@Misc$lbinmid)/rowSums(RCMdata@IAL[, , ", sur, "], na.rm = TRUE)"),
            "ylim <- c(0.9, 1.1) * range(c(MLpred, MLobs), na.rm = TRUE)",
            "matplot(Year, MLpred, type = \"l\", col = scenario$col, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Mean length\", ylim = ylim)",
@@ -302,15 +302,15 @@ rmd_RCM_index_output <- function(sur, s_name) {
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) age composition from ", s_name[sur], ".\"}"),
            paste0("if(length(RCMdata@IAA) && any(RCMdata@IAA[, , ", sur, "] > 0, na.rm = TRUE)) {"),
-           paste0("pred_sCAA <- lapply(report_list, function(x) x$IAA[,, ", sur, "]) %>% simplify2array() %>% aperm(perm = c(3, 1, 2))"),
-           paste0("plot_composition_RCM(Year, pred_sCAA, RCMdata@IAA[, , ", sur, "], ages = age, dat_col = scenario$col)"),
+           paste0("pred_IAA <- sapply(report_list, function(x) x$IAA[, , ", sur, "], simplify = \"array\") %>% aperm(perm = c(3, 1, 2))"),
+           paste0("plot_composition_RCM(Year, pred_IAA, RCMdata@IAA[, , ", sur, "], ages = age, dat_col = scenario$col)"),
            "}",
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) length composition from ", s_name[sur], ".\"}"),
            paste0("if(length(RCMdata@IAL) && any(RCMdata@IAL[, , ", sur, "] > 0, na.rm = TRUE)) {"),
-           paste0("pred_sCAL <- lapply(report_list, function(x) x$IAL[,, ", sur, "]) %>% simplify2array() %>% aperm(perm = c(3, 1, 2))"),
-           paste0("plot_composition_RCM(Year, pred_sCAL, RCMdata@IAL[, , ", sur, "], CAL_bins = RCMdata@Misc$lbinmid, dat_col = scenario$col)"),
+           paste0("pred_IAL <- sapply(report_list, function(x) x$IAL[, , ", sur, "], simplify = \"array\") %>% aperm(perm = c(3, 1, 2))"),
+           paste0("plot_composition_RCM(Year, pred_IAL, RCMdata@IAL[, , ", sur, "], CAL_bins = RCMdata@Misc$lbinmid, dat_col = scenario$col)"),
            "}",
            "```\n"
   )
@@ -319,14 +319,14 @@ rmd_RCM_index_output <- function(sur, s_name) {
 
 rmd_RCM_initD <- function(fig.cap = "Histogram of initial depletion among all simulations.") {
   c(paste0("```{r, fig.cap = \"", fig.cap, "\"}"),
-    "initD <- vapply(report_list, function(x) x$E[1]/x$E0[1], numeric(1))",
+    "initD <- vapply(report_list, function(x) x$E[1]/x$E0_SR, numeric(1))",
     "hist(initD, main = \"\", xlab = \"Initial depletion\")",
     "```\n")
 }
 
 rmd_RCM_R_output <- function() {
   c("```{r, fig.cap = \"Estimated recruitment among all simulations.\"}",
-    "R_out <- do.call(cbind, lapply(report_list, getElement, \"R\"))",
+    "R_out <- sapply(report_list, getElement, \"R\")",
     "matplot(Yearplusone, R_out, ylim = c(0, 1.1 * max(R_out, na.rm = TRUE)), type = \"l\", col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Recruitment\")",
     "abline(h = 0, col = \"grey\")",
     "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -335,21 +335,21 @@ rmd_RCM_R_output <- function() {
 
 rmd_RCM_SSB_output <- function() {
   c("```{r, fig.cap = \"Estimated spawning biomass among all simulations.\"}",
-    "E <- do.call(cbind, lapply(report_list, getElement, \"E\"))",
+    "E <- sapply(report_list, getElement, \"E\")",
     "matplot(Yearplusone, E, ylim = c(0, 1.1 * max(E, na.rm = TRUE)), type = \"l\", col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Spawning biomass\")",
     "abline(h = 0, col = \"grey\")",
     "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
     "```\n",
     "",
     "```{r, fig.cap = \"Estimated spawning depletion among all simulations. Unfished spawning biomass is the value calculated from first year life history parameters.\"}",
-    "E_E0 <- do.call(cbind, lapply(report_list, function(x) x$E/x$E0_SR))",
+    "E_E0 <- sapply(report_list, function(x) x$E/x$E0_SR)",
     "matplot(Yearplusone, E_E0, ylim = c(0, 1.1 * max(E_E0, na.rm = TRUE)), type = \"l\", col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = \"Spawning depletion\")",
     "abline(h = 0, col = \"grey\")",
     "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
     "```\n",
     "",
     "```{r, fig.cap = \"Dynamic SSB0 among all simulations. Model is re-run assuming no historical catches.\"}",
-    "dyn_SSB0 <- do.call(cbind, lapply(report_list, function(x) x$dynamic_SSB0))",
+    "dyn_SSB0 <- sapply(report_list, function(x) x$dynamic_SSB0)",
     "matplot(Yearplusone, dyn_SSB0, ylim = c(0, 1.1 * max(dyn_SSB0, na.rm = TRUE)), type = \"l\", col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd, xlab = \"Year\", ylab = expression(Dynamic~~SSB[0]))",
     "abline(h = 0, col = \"grey\")",
     "if(!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -358,7 +358,7 @@ rmd_RCM_SSB_output <- function() {
 
 rmd_log_rec_dev <- function() {
   c("```{r, fig.cap = \"Estimated recruitment deviations among all simulations.\"}",
-    "log_rec_dev2 <- do.call(cbind, lapply(report_list, getElement, \"log_rec_dev\"))",
+    "log_rec_dev2 <- sapply(report_list, getElement, \"log_rec_dev\")",
     "matplot(Year, log_rec_dev2, type = \"n\", xlab = \"Year\", ylab = \"log-recruitment deviations\")",
     "abline(h = 0, col = \"grey\")",
     "matlines(Year, log_rec_dev2, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -368,7 +368,7 @@ rmd_log_rec_dev <- function() {
 
 rmd_RCM_SPR <- function() {
   c("```{r, fig.cap = \"Equilibrium spawning potential ratio (SPR) calculated from the biological parameters and F-at-age in the corresponding year for all simulations.\"}",
-    "SPR_eq <- do.call(cbind, lapply(report_list, getElement, \"SPR_eq\"))",
+    "SPR_eq <- sapply(report_list, getElement, \"SPR_eq\")",
     "matplot(Year, SPR_eq, type = \"n\", ylim = c(0, 1), xlab = \"Year\", ylab = \"Equilibrium SPR\")",
     "abline(h = 0, col = \"grey\")",
     "matlines(Year, SPR_eq, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -376,7 +376,7 @@ rmd_RCM_SPR <- function() {
     "```\n",
     "",
     "```{r, fig.cap = \"Dynamic spawning potential ratio (SPR) calculated from the biological parameters and cumulative survival of the cohorts in the corresponding year for all simulations.\"}",
-    "SPR_dyn <- do.call(cbind, lapply(report_list, getElement, \"SPR_dyn\"))",
+    "SPR_dyn <- sapply(report_list, getElement, \"SPR_dyn\")",
     "matplot(Year, SPR_dyn, type = \"n\", ylim = c(0, 1), xlab = \"Year\", ylab = \"Dynamic SPR\")",
     "abline(h = 0, col = \"grey\")",
     "matlines(Year, SPR_dyn, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
@@ -551,14 +551,16 @@ plot_composition_RCM <- function(Year, fit, dat = NULL, CAL_bins = NULL, ages =
   }
   
   # Annual comps (fit vs. dat if available)
-  # Dim of
-  fit_plot <- fit
-  dat_plot <- dat
   if(annual_yscale == "proportions") {
-    for(i in 1:length(Year)) {
-      fit_plot[, i, ] <- fit[, i, ]/rowSums(fit[, i, ])
-      if(!is.null(dat)) dat_plot[i, ] <- dat[i, ]/sum(dat[i, ])
+    fit_plot <- apply(fit, 1:2, function(x) x/sum(x)) %>% aperm(c(2, 3, 1))
+    if(!is.null(dat)) {
+      dat_plot <- dat/rowSums(dat)
+    } else {
+      dat_plot <- dat
     }
+  } else {
+    fit_plot <- fit
+    dat_plot <- dat
   }
   ylim <- c(0, 1.1 * max(fit_plot, dat_plot, na.rm = TRUE))
   yaxp <- c(0, max(pretty(ylim, n = 4)), 4)
