@@ -93,11 +93,12 @@ SSS <- function(x = 1, Data, dep = 0.4, SR = c("BH", "Ricker"),
                abs_I = 0, nsurvey = 1, LWT = 1,
                CAA_hist = matrix(0, n_y, max_age), CAA_n = rep(0, n_y), 
                CAL_hist = matrix(0, n_y, 1), CAL_n = rep(0, n_y),
-               n_y = n_y, n_age = n_age, n_bin = 1L, M_data = M,
+               n_y = n_y, n_age = n_age, n_bin = 1L, M_data = 1,
                weight = Wa, PLA = matrix(1, n_age, 1), mat = mat_age, vul_type = "logistic",
                SR_type = SR, comp_dist = "multinomial", catch_eq = catch_eq,
                est_early_rec_dev = rep(0, n_age - 1), est_rec_dev = rep(0, n_y), yindF = 0,
                tv_M = "none", M_bounds = c(0, 1e4), use_prior = rep(0, 4), prior_dist = matrix(NA, 4, 2))
+  if(any(names(dots) == "M_at_age") && dots$M_at_age) data$M_data <- M
 
   # Starting values
   params <- list()
